searchData={"items":[{"type":"module","title":"Phoenix.Ecto","doc":"Integrates Phoenix with Ecto.\n\nIt implements many protocols that make it easier to use\nEcto with Phoenix either when working with HTML or JSON.","ref":"Phoenix.Ecto.html"},{"type":"function","title":"Phoenix.Ecto.start/2","doc":"","ref":"Phoenix.Ecto.html#start/2"},{"type":"module","title":"Phoenix.Ecto.CheckRepoStatus","doc":"A plug that does some checks on your application repos.\n\nChecks if the storage is up (database is created) or if there are any pending migrations.\nBoth checks can raise an error if the conditions are not met.","ref":"Phoenix.Ecto.CheckRepoStatus.html"},{"type":"module","title":"Plug options - Phoenix.Ecto.CheckRepoStatus","doc":"* `:otp_app` - name of the application which the repos are fetched from\n  * `:migration_paths` - a function that accepts a repo and returns a migration directory, or a list of migration directories, that is used to check for pending migrations\n  * `:migration_lock` - the locking strategy used by the Ecto Adapter when checking for pending migrations. Set to `false` to disable migration locks.\n  * `:prefix` - the prefix used to check for pending migrations.\n  * `:skip_table_creation` - Ecto will not try to create the `schema_migrations` table automatically.  This is useful if you are connecting as a DB user without create permissions","ref":"Phoenix.Ecto.CheckRepoStatus.html#module-plug-options"},{"type":"function","title":"Phoenix.Ecto.CheckRepoStatus.call/2","doc":"","ref":"Phoenix.Ecto.CheckRepoStatus.html#call/2"},{"type":"function","title":"Phoenix.Ecto.CheckRepoStatus.init/1","doc":"","ref":"Phoenix.Ecto.CheckRepoStatus.html#init/1"},{"type":"module","title":"Phoenix.Ecto.SQL.Sandbox","doc":"A plug to allow concurrent, transactional acceptance tests with\n[`Ecto.Adapters.SQL.Sandbox`](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html).","ref":"Phoenix.Ecto.SQL.Sandbox.html"},{"type":"module","title":"Example - Phoenix.Ecto.SQL.Sandbox","doc":"This plug should only be used during tests. First, set a flag to\nenable it in `config/test.exs`:\n\n    config :your_app, sql_sandbox: true\n\nAnd use the flag to conditionally add the plug to `lib/your_app/endpoint.ex`:\n\n    if Application.compile_env(:your_app, :sql_sandbox) do\n      plug Phoenix.Ecto.SQL.Sandbox\n    end\n\nIt's important that this is at the top of `endpoint.ex`, before any other plugs.\n\nThen, within an acceptance test, checkout a sandboxed connection as before.\nUse `metadata_for/2` helper to get the session metadata to that will allow access\nto the test's connection. In general lines, you would write this:\n\n    setup tags do\n      pid = Ecto.Adapters.SQL.Sandbox.start_owner!(YourApp.Repo, shared: not tags[:async])\n      on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n      metadata_header = Phoenix.Ecto.SQL.Sandbox.metadata_for(YourApp.Repo, pid)\n      # pass the metadata to the acceptance test library\n      :ok\n    end\n\nYou can follow the instructions for Wallaby [here](https://hexdocs.pm/wallaby/readme.html#phoenix).","ref":"Phoenix.Ecto.SQL.Sandbox.html#module-example"},{"type":"module","title":"Acceptance tests with channels - Phoenix.Ecto.SQL.Sandbox","doc":"To support channels, in addition the above, you need to make it so each channel\nis allowed within the sandbox. The first step is to access the relevant header\nmetadata.\n\nTo do so, you must declare that you want to pass connection information to your\nsocket. This is typically the user agent header, but it can be a custom x-header\ntoo:\n\n    socket \"/path\", Socket,\n      websocket: [connect_info: [:user_agent, …]]\n\n    socket \"/path\", Socket,\n      websocket: [connect_info: [:x_headers, …]]\n\nNow use the `c:Phoenix.Socket.connect/3` callback to access the header and\nstore it in the socket:\n\n    # user_socket.ex\n    def connect(_params, socket, connect_info) do\n      {:ok, assign(socket, :phoenix_ecto_sandbox, connect_info.user_agent)}\n    end\n\nOr if you are using a custom header:\n\n    Enum.find_value(connect_info.x_headers, fn\n      {\"x-my-custom-header\", val} -> val\n      _ -> nil\n    end)\n\nThis stores the value on the socket, so it can be available to all of your\nchannels for allowing the sandbox.\n\n    # room_channel.ex\n    def join(\"room:lobby\", _payload, socket) do\n      allow_ecto_sandbox(socket)\n      {:ok, socket}\n    end\n\n    # This is a great function to extract to a helper module\n    defp allow_ecto_sandbox(socket) do\n      Phoenix.Ecto.SQL.Sandbox.allow(\n        socket.assigns.phoenix_ecto_sandbox,\n        Ecto.Adapters.SQL.Sandbox\n      )\n    end\n\n`allow/2` needs to be manually called once for each channel, at best directly\nat the start of `c:Phoenix.Channel.join/3`.","ref":"Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-channels"},{"type":"module","title":"Acceptance tests with LiveViews - Phoenix.Ecto.SQL.Sandbox","doc":"LiveViews can be supported in a similar fashion to channels. First declare the\n`:user_agent` (or a custom header) in your live socket configuration in `endpoint.ex`:\n\n    socket \"/live\", Phoenix.LiveView.Socket,\n      websocket: [connect_info: [:user_agent, session: @session_options]]\n\nNow you can use the `on_mount/4` callback to check the header and assign the sandbox:\n\n    defmodule MyApp.LiveAcceptance do\n      import Phoenix.LiveView\n      import Phoenix.Component\n\n      def on_mount(:default, _params, _session, socket) do\n        socket =\n          assign_new(socket, :phoenix_ecto_sandbox, fn ->\n            if connected?(socket), do: get_connect_info(socket, :user_agent)\n          end)\n\n        metadata = socket.assigns.phoenix_ecto_sandbox\n        Phoenix.Ecto.SQL.Sandbox.allow(metadata, Ecto.Adapters.SQL.Sandbox)\n        {:cont, socket}\n      end\n    end\n    \nNow, in your `my_app_web.ex` file, you can invoke this callback for all of your\nLiveViews if the sandbox configuration, defined at the beginning of the\ndocumentation, is enabled:\n\n    def live_view do\n      quote do\n        use Phoenix.LiveView\n        # ...\n        \n        if Application.compile_env(:your_app, :sql_sandbox) do\n          on_mount MyApp.LiveAcceptance\n        end\n        \n        # ...\n      end\n    end\n\nIf you have `on_mount` hooks in `live_session` defined in your `router.ex`\n(for example, routes requiring authentication after running `mix phx.gen.auth`\nto generate your authentication system), make sure the `MyApp.LiveAcceptance`\nhook runs before, so following hooks have access to the Ecto Sandbox:\n\n    live_session :require_authenticated_user,\n      on_mount:\n        if(Application.compile_env(:your_app, :sql_sandbox),\n          do: [MyAppWeb.AcceptanceHook],\n          else: []\n        ) ++ [{MyAppWeb.UserAuth, :ensure_authenticated}] do\n      live \"/users/settings\", UserSettingsLive, :edit\n      live \"/users/settings/confirm_email/:token\", UserSettingsLive, :confirm_email\n    end","ref":"Phoenix.Ecto.SQL.Sandbox.html#module-acceptance-tests-with-liveviews"},{"type":"module","title":"Concurrent end-to-end tests with external clients - Phoenix.Ecto.SQL.Sandbox","doc":"Concurrent and transactional tests for external HTTP clients is supported,\nallowing for complete end-to-end tests. This is useful for cases such as\nJavaScript test suites for single page applications that exercise the\nPhoenix endpoint for end-to-end test setup and teardown. To enable this,\nyou can expose a sandbox route on the `Phoenix.Ecto.SQL.Sandbox` plug by\nproviding the `:at`, and `:repo` options. For example:\n\n    plug Phoenix.Ecto.SQL.Sandbox,\n      at: \"/sandbox\",\n      repo: MyApp.Repo,\n      timeout: 15_000 # the default\n\nThis would expose a route at `\"/sandbox\"` for the given repo where\nexternal clients send POST requests to spawn a new sandbox session,\nand DELETE requests to stop an active sandbox session. By default,\nthe external client is expected to pass up the `\"user-agent\"` header\ncontaining serialized sandbox metadata returned from the POST request,\nbut this value may customized with the `:header` option.\n\nFinally, make sure your repository mode is set either to `:manual`\nor `{:shared, self()}` before the external client starts. This is\ntypically done by default in your `test/test_helper.exs`, but you\nmay need to do it explicitly depending on your setup:\n\n    Ecto.Adapters.SQL.Sandbox.mode(MyApp.Repo, :manual)","ref":"Phoenix.Ecto.SQL.Sandbox.html#module-concurrent-end-to-end-tests-with-external-clients"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.allow/2","doc":"Decodes the given metadata and allows the current process\nunder the given sandbox.","ref":"Phoenix.Ecto.SQL.Sandbox.html#allow/2"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.decode_metadata/1","doc":"Decodes encoded metadata back into map generated from `metadata_for/2`.","ref":"Phoenix.Ecto.SQL.Sandbox.html#decode_metadata/1"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.encode_metadata/1","doc":"Encodes metadata generated by `metadata_for/2` for client response.","ref":"Phoenix.Ecto.SQL.Sandbox.html#encode_metadata/1"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.metadata_for/3","doc":"Returns metadata to establish a sandbox for.\n\nThe metadata is then passed via user-agent/headers to browsers.\nUpon request, the `Phoenix.Ecto.SQL.Sandbox` plug will decode\nthe header and allow the request process under the sandbox.","ref":"Phoenix.Ecto.SQL.Sandbox.html#metadata_for/3"},{"type":"function","title":"Options - Phoenix.Ecto.SQL.Sandbox.metadata_for/3","doc":"* `:trap_exit` - if the browser being used for integration\n    testing navigates away from a page or aborts a AJAX request\n    while the request process is talking to the database, it\n    will corrupt the database connection and make the test fail.\n    Therefore, to avoid intermittent tests, we recommend trapping\n    exits in the request process, so all database connections shut\n    down cleanly. You can disable this behaviour by setting the\n    option to false.","ref":"Phoenix.Ecto.SQL.Sandbox.html#metadata_for/3-options"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.start_child/2","doc":"Spawns a sandbox session to checkout a connection for a remote client.","ref":"Phoenix.Ecto.SQL.Sandbox.html#start_child/2"},{"type":"function","title":"Examples - Phoenix.Ecto.SQL.Sandbox.start_child/2","doc":"iex> {:ok, _owner_pid, metadata} = start_child(MyApp.Repo)","ref":"Phoenix.Ecto.SQL.Sandbox.html#start_child/2-examples"},{"type":"function","title":"Phoenix.Ecto.SQL.Sandbox.stop/1","doc":"Stops a sandbox session holding a connection for a remote client.","ref":"Phoenix.Ecto.SQL.Sandbox.html#stop/1"},{"type":"function","title":"Examples - Phoenix.Ecto.SQL.Sandbox.stop/1","doc":"iex> {:ok, owner_pid, metadata} = start_child(MyApp.Repo)\n    iex> :ok = stop(owner_pid)","ref":"Phoenix.Ecto.SQL.Sandbox.html#stop/1-examples"},{"type":"exception","title":"Phoenix.Ecto.PendingMigrationError","doc":"","ref":"Phoenix.Ecto.PendingMigrationError.html"},{"type":"function","title":"Phoenix.Ecto.PendingMigrationError.message/1","doc":"","ref":"Phoenix.Ecto.PendingMigrationError.html#message/1"},{"type":"exception","title":"Phoenix.Ecto.StorageNotCreatedError","doc":"","ref":"Phoenix.Ecto.StorageNotCreatedError.html"},{"type":"function","title":"Phoenix.Ecto.StorageNotCreatedError.message/1","doc":"","ref":"Phoenix.Ecto.StorageNotCreatedError.html#message/1"},{"type":"extras","title":"Phoenix/Ecto","doc":"# Phoenix/Ecto\n\n[![Action Status](https://github.com/phoenixframework/phoenix_ecto/workflows/CI/badge.svg)](https://github.com/phoenixframework/phoenix_ecto/actions?query=workflow%3ACI)\n\nA project that integrates [Phoenix](http://github.com/phoenixframework/phoenix) with [Ecto](http://github.com/elixir-lang/ecto), implementing all relevant protocols.","ref":"main.html"},{"type":"extras","title":"Installation - Phoenix/Ecto","doc":"You can install `phoenix_ecto` by adding it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [{:phoenix_ecto, \"~> 4.0\"}]\nend\n```","ref":"main.html#installation"},{"type":"extras","title":"The Phoenix <-> Ecto integration - Phoenix/Ecto","doc":"Thanks to Elixir protocols, the integration between Phoenix and Ecto is simply a matter of implementing a handful of protocols. We provide the following implementations:\n\n  * `Phoenix.HTML.FormData` protocol for `Ecto.Changeset`\n  * `Phoenix.HTML.Safe` protocol for `Decimal`\n  * `Plug.Exception` protocol for the relevant Ecto exceptions","ref":"main.html#the-phoenix-ecto-integration"},{"type":"extras","title":"Concurrent browser tests - Phoenix/Ecto","doc":"This library also provides a plug called `Phoenix.Ecto.SQL.Sandbox` that allows developers to run acceptance tests powered by headless browsers such as ChromeDriver and Selenium concurrently. If you are not familiar with Ecto's SQL sandbox, we recommend you to first get acquainted with it by [reading `Ecto.Adapters.SQL.Sandbox` documentation](https://hexdocs.pm/ecto_sql/Ecto.Adapters.SQL.Sandbox.html).\n\nTo enable concurrent acceptance tests, make sure you are using PostgreSQL and follow the instructions below:\n\n1. Set a flag to enable the sandbox in `config/test.exs`:\n\n    ```elixir\n    config :your_app, sql_sandbox: true\n    ```\n\n2. And use the flag to conditionally add the plug to `lib/your_app/endpoint.ex`:\n\n    ```elixir\n    if Application.get_env(:your_app, :sql_sandbox) do\n      plug Phoenix.Ecto.SQL.Sandbox\n    end\n    ```\n\n    Make sure that this is placed **before** the line `plug YourApp.Router` (or any other plug that may access the database).\n\nYou can now checkout a sandboxed connection and pass the connection information to an acceptance testing tool like [Hound](https://github.com/hashnuke/hound) or [Wallaby](https://github.com/elixir-wallaby/wallaby).","ref":"main.html#concurrent-browser-tests"},{"type":"extras","title":"Hound - Phoenix/Ecto","doc":"To write concurrent acceptance tests with Hound, first add it as a dependency to your `mix.exs`:\n\n```elixir\n{:hound, \"~> 1.0\"}\n```\n\nMake sure to start it at the top of your `test/test_helper.exs`:\n\n```elixir\n{:ok, _} = Application.ensure_all_started(:hound)\n```\n\nThen add the following to your test case (or case template):\n\n```elixir\nuse Hound.Helpers\n\nsetup tags do\n  pid = Ecto.Adapters.SQL.Sandbox.start_owner!(YourApp.Repo, shared: not tags[:async])\n  on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n  metadata = Phoenix.Ecto.SQL.Sandbox.metadata_for(YourApp.Repo, pid)\n  Hound.start_session(metadata: metadata)\n  :ok\nend\n```\n\nHound supports multiple drivers like Chrome, Firefox, etc but it does not support concurrent tests under PhantomJS (the default).","ref":"main.html#hound"},{"type":"extras","title":"Wallaby - Phoenix/Ecto","doc":"To write concurrent acceptance tests with Wallaby, first add it as a dependency to your `mix.exs`:\n\n```elixir\n{:wallaby, \"~> 0.25\", only: :test}\n```\n\nWallaby can take care of setting up the Ecto Sandbox for you if you use `use Wallaby.Feature` in your test module.\n\n```elixir\ndefmodule MyAppWeb.PageFeature do\n  use ExUnit.Case, async: true\n  use Wallaby.Feature\n\n  feature \"shows some text\", %{session: session} do\n    session\n    |> visit(\"/home\")\n    |> assert_text(\"Hello world!\")\n  end\nend\n```\n\nIf you don't `use Wallaby.Feature`, you can add the following to your test case (or case template):\n\n```elixir\nuse Wallaby.DSL\n\nsetup tags do\n  pid = Ecto.Adapters.SQL.Sandbox.start_owner!(YourApp.Repo, shared: not tags[:async])\n  on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n  metadata = Phoenix.Ecto.SQL.Sandbox.metadata_for(YourApp.Repo, pid)\n  {:ok, session} = Wallaby.start_session(metadata: metadata)\nend\n```\n\nWallaby currently supports ChromeDriver and Selenium, allowing testing in almost any browser.","ref":"main.html#wallaby"},{"type":"extras","title":"Configuration - Phoenix/Ecto","doc":"The `Plug.Exception` implementations for Ecto exceptions may be disabled by including the error in the mix configuration.\n\n```elixir\nconfig :phoenix_ecto,\n  exclude_ecto_exceptions_from_plug: [Ecto.NoResultsError]\n```","ref":"main.html#configuration"},{"type":"extras","title":"Copyright and License - Phoenix/Ecto","doc":"Copyright (c) 2015, Chris McCord.\n\nPhoenix/Ecto source code is licensed under the [MIT License](https://github.com/phoenixframework/phoenix_ecto/blob/master/LICENSE).","ref":"main.html#copyright-and-license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}